import matplotlib.pyplot as plt
import math
import cv2 as cv 
import numpy as np
# Stop command is by ~
class TextStega:
    
    '''
    Methods
    -------
    Public:
        encode : encodes the text to the image
        decode : decodes the text from the image
    '''
    
    def decode(self, path):    
        """
        Input parameter 
        path(str) : Path of the image from which the encoded message is to be decoded

        Output
        text(str) : Decoded message from the image  

        Example
        msg = decode(path)

        Decode function decodes all the characters till it encounters ~(tilda) character.  
        """

        self.img = cv.imread(path)

        self.img = cv.cvtColor(self.img, cv.COLOR_BGR2RGB)
        self.img_str = np.reshape(self.img, (np.product(self.img.shape)))

        self.text = ""

        for i in range(int(len(self.img_str)/8)):
            self.bit = self.img_str[8*i:8*(i+1)]
            #print(bit.shape)
            #print(bit)
            self.bit = np.remainder(self.bit, 2)
            #print(bit.shape)
            #print(bit)
            self.bit = ''.join(map(str, self.bit))
            #print(bit)

            self.ascii_character = chr(int(self.bit, 2))
            #print(ascii_character)
            if self.ascii_character == '~' :
                break
            else:
                self.text = self.text + self.ascii_character
            i+=8
        return self.text


    def encode(path, text):
        '''
        Input parameter: 
        path(str) : Path of the image to which the message is to be encoded
        text(str) : Message to be encoded

        Output:
        final_image(numpy array) :  New image on which the message is encoded

        Example:
        img = encode(path, text)

        Stop character is ~(tilda). Encode function automatically adds this character at the end of the user input string. No need to specifically add this character. 

        '''

        self.img = cv.imread(path)

        self.limit = math.floor((self.img.shape[0]*self.img.shape[1]*self.img.shape[2]-8)/8)

        if len(text) <= (self.img.shape[0]*self.img.shape[1]*self.img.shape[2]-8)/8 :

            self.img = cv.cvtColor(self.img, cv.COLOR_BGR2RGB)
            self.img_str = np.reshape(self.img, (np.product(self.img.shape)))

            self.bit_str=""
            self.a = ' '.join(format(ord(x), 'b') for x in text)
            self.LL = list(self.a.split(' '))
            for i, bit in enumerate(LL):
                self.bit_str = self.bit_str + bit.zfill(8)

            self.bit_str = self.bit_str + "01111110"
            #print(bit_str)

            for i in range(len(self.bit_str)):
                #print(bit_str[i],img_str[i])

                if self.bit_str[i] == '0':
                    if self.img_str[i]%2 != 0:

                        self.img_str[i] = self.img_str[i] + 1
                    #print(bit_str[i],img_str[i])
                else:
                    if self.img_str[i]%2 == 0:

                        self.img_str[i] = self.img_str[i] + 1
                    #print(bit_str[i],img_str[i])

            self.final_image = np.reshape(self.img_str , self.img.shape)

            cv.imwrite("Encoded_image.png", cv.cvtColor(self.final_image, cv.COLOR_RGB2BGR))
            print(r"Encoded image is saved as 'Encoded_image.png'")
            return self.final_image

        else:
            print("Try again with higher resolution image!\nNo.of characters to encode exceed maximum capacity of the image\n")
            print("\nMaximum Characters allowed are : ",self.limit)
            
            return None



    


"""
This module implements text steganography
Author: Atharva Vidwans
Github: https://github.com/Atharva-Vidwans
LinkedIn: https://www.linkedin.com/in/atharva-vidwans-62739b169/
Medium: https://medium.com/@atharvavidwans
Website: https://atharva-vidwans.github.io/Website/ 

"""

__author__ = "Atharva Vidwans"
__email__ = "atharvavidwans@gmail.com"
__status__ = "stable"
__version__= "0.0.8" 

from TextStega.stg import TextStega 


from setuptools import setup, find_packages

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()


classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Education',
    'Operating System :: Microsoft :: Windows :: Windows 10',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3'
]

setup(
    name='TextStega',
    version='0.0.8',
    description='A package for text steganography',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='',
    author='Atharva Vidwans',
    author_email='atharvavidwans@gmail.com',
    license='MIT',
    classifiers=classifiers,
    kerwords='steganography',
    packages=find_packages(),
    install_requires=['numpy',
                      'matplotlib',
                      'opencv-python']
)
    